// • ▌ ▄ ·.  ▄▄▄·  ▄▄ • ▪   ▄▄· ▄▄▄▄·  ▄▄▄·  ▐▄▄▄  ▄▄▄ .
// ·██ ▐███▪▐█ ▀█ ▐█ ▀ ▪██ ▐█ ▌▪▐█ ▀█▪▐█ ▀█ •█▌ ▐█▐▌·
// ▐█ ▌▐▌▐█·▄█▀▀█ ▄█ ▀█▄▐█·██ ▄▄▐█▀▀█▄▄█▀▀█ ▐█▐ ▐▌▐▀▀▀
// ██ ██▌▐█▌▐█ ▪▐▌▐█▄▪▐█▐█▌▐███▌██▄▪▐█▐█ ▪▐▌██▐ █▌▐█▄▄▌
// ▀▀  █▪▀▀▀ ▀  ▀ ·▀▀▀▀ ▀▀▀·▀▀▀ ·▀▀▀▀  ▀  ▀ ▀▀  █▪ ▀▀▀
//      Magicbane Emulator Project © 2013 - 2022
//                www.magicbane.com

package engine.gameManager;

import engine.Enum;
import engine.loot.*;
import engine.net.DispatchMessage;
import engine.net.client.msg.ErrorPopupMsg;
import engine.net.client.msg.chat.ChatSystemMsg;
import engine.objects.*;
import engine.server.MBServerStatics;
import org.pmw.tinylog.Logger;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.concurrent.ThreadLocalRandom;

/**
 * Class contains static methods for data from Magicbane's loot tables
 */
public enum LootManager {

    LOOTMANAGER;

    // Newer tables

    public static HashMap<Integer, ArrayList<GenTableEntry>> _genTables = new HashMap<>();
    public static HashMap<Integer, ArrayList<ItemTableEntry>> _itemTables = new HashMap<>();
    public static HashMap<Integer, ArrayList<ModTableEntry>> _modTables = new HashMap<>();
    public static HashMap<Integer, ArrayList<ModTypeTableEntry>> _modTypeTables = new HashMap<>();

    public static ArrayList<Integer> vorg_ha_uuids = new ArrayList<>(Arrays.asList(new Integer[]{27580, 27590, 188500, 188510, 188520, 188530, 188540, 188550, 189510}));
    public static ArrayList<Integer> vorg_ma_uuids = new ArrayList<>(Arrays.asList(new Integer[]{27570,188900,188910,188920,188930,188940,188950,189500}));
    public static ArrayList<Integer> vorg_la_uuids = new ArrayList<>(Arrays.asList(new Integer[]{27550,27560,189100,189110,189120,189130,189140,189150}));
    public static ArrayList<Integer> vorg_cloth_uuids = new ArrayList<>(Arrays.asList(new Integer[]{27600,188700,188720,189550,189560}));

    // Drop Rates

    public static float NORMAL_DROP_RATE;
    public static float NORMAL_EXP_RATE;
    public static float NORMAL_GOLD_RATE;
    public static float HOTZONE_DROP_RATE;
    public static float HOTZONE_EXP_RATE;
    public static float HOTZONE_GOLD_RATE;
    public static HashMap<Integer, ArrayList<BootySetEntry>> _bootySetMap = new HashMap<>();

    // Bootstrap routine to initialize the Loot Manager

    public static void init() {

        // Load loot tables from database.

        _genTables = DbManager.LootQueries.LOAD_GEN_ITEM_TABLES();
        _itemTables = DbManager.LootQueries.LOAD_ITEM_TABLES();
        _modTables = DbManager.LootQueries.LOAD_MOD_TABLES();
        _modTypeTables = DbManager.LootQueries.LOAD_MOD_TYPE_TABLES();

        // Cache drop rate values from Config manager.

        NORMAL_DROP_RATE = Float.parseFloat(ConfigManager.MB_NORMAL_DROP_RATE.getValue());
        NORMAL_EXP_RATE = Float.parseFloat(ConfigManager.MB_NORMAL_EXP_RATE.getValue());
        NORMAL_GOLD_RATE = Float.parseFloat(ConfigManager.MB_NORMAL_GOLD_RATE.getValue());
        HOTZONE_DROP_RATE = Float.parseFloat(ConfigManager.MB_HOTZONE_DROP_RATE.getValue());
        HOTZONE_EXP_RATE = Float.parseFloat(ConfigManager.MB_HOTZONE_EXP_RATE.getValue());
        HOTZONE_GOLD_RATE = Float.parseFloat(ConfigManager.MB_HOTZONE_GOLD_RATE.getValue());

    }

    public static void GenerateMobLoot(Mob mob) {

        //iterate the booty sets

        if (mob.getMobBase().bootySet != 0 && _bootySetMap.containsKey(mob.getMobBase().bootySet) == true)
            RunBootySet(_bootySetMap.get(mob.getMobBase().bootySet), mob);

        if (mob.bootySet != 0 && _bootySetMap.containsKey(mob.bootySet) == true)
            RunBootySet(_bootySetMap.get(mob.bootySet), mob);

        //lastly, check mobs inventory for godly or disc runes to send a server announcement
            for (Item it : mob.getInventory()) {

                ItemBase ib = it.getItemBase();
                if(ib == null)
                    break;
                if (ib.getName().toLowerCase().contains("of the gods")) {
                    ChatSystemMsg chatMsg = new ChatSystemMsg(null, mob.getName() + " in " + mob.getParentZone().getName() + " has found the " + ib.getName() + ". Are you tough enough to take it?");
                    chatMsg.setMessageType(10);
                    chatMsg.setChannel(Enum.ChatChannelType.SYSTEM.getChannelID());
                    DispatchMessage.dispatchMsgToAll(chatMsg);
                }
            }

    }

    private static void RunBootySet(ArrayList<BootySetEntry> entries, Mob mob) {
        mob.runeCounter++;
        mob.contractCounter++;
        float dropRate = NORMAL_DROP_RATE;
        //roll the geenric world drop table
        if(mob.parentZone.getSafeZone() == 0) {
            GenerateLootDrop(mob, 1300);
            if(ThreadLocalRandom.current().nextInt(1, 10000) == 5000) {
                MobLoot extraLoot = rollForGlass(mob);
                if (extraLoot != null) {
                    mob.getCharItemManager().addItemToInventory(extraLoot);
                }
            }
        }

        // Iterate all entries in this bootySet and process accordingly
        for (BootySetEntry bse : entries) {
            switch (bse.bootyType) {
                case "GOLD":
                    GenerateGoldDrop(mob, bse);
                    break;
                case "LOOT":
                        if (ThreadLocalRandom.current().nextInt(1, 100 + 1) < (bse.dropChance * dropRate))
                            GenerateLootDrop(mob, bse.genTable);  //generate normal loot drop
                    if(ThreadLocalRandom.current().nextInt(1,100) == 50){
                        MobLoot extraLoot = rollForContract(bse.genTable, mob);
                        if (extraLoot != null) {
                            mob.getCharItemManager().addItemToInventory(extraLoot);
                        }
                    }
                    if(ThreadLocalRandom.current().nextInt(1,100) == 50){
                        MobLoot extraLoot = rollForRune(bse.genTable, mob);
                        if (extraLoot != null) {
                            mob.getCharItemManager().addItemToInventory(extraLoot);
                        }
                    }
                    break;
                case "ITEM":
                    GenerateInventoryDrop(mob, bse);
                    break;
            }
        }
        MobLoot specialDrop = null;
        switch(mob.getObjectUUID()) {
            case 22595://elf 1
                specialDrop = new MobLoot(mob,ItemBase.getItemBase(252134),true);
                mob.setFirstName("Melandrach The Blood-Mage");
                break;
            case 22432: //elf 2
                specialDrop = new MobLoot(mob,ItemBase.getItemBase(252135),true);
                mob.setFirstName("Kyrtaar The Blood-Mage");
                break;
            case 22537: //elf 3
                specialDrop = new MobLoot(mob,ItemBase.getItemBase(252136),true);
                mob.setFirstName("Vamir The Blood-Mage");
                break;
            case 16387: //human 4 DONE
                specialDrop = new MobLoot(mob,ItemBase.getItemBase(252129),true);
                mob.setFirstName("Alatar The Blood-Mage");
                break;
            case 32724:// human 5 GOOD
                specialDrop = new MobLoot(mob,ItemBase.getItemBase(252130),true);
                mob.setFirstName("Elphaba The Blood-Mage");
                break;
            case 23379: //human 1 GOOD
                specialDrop = new MobLoot(mob,ItemBase.getItemBase(252131),true);
                mob.setFirstName("Bavmorda The Blood-Mage");
                break;
            case 10826: //human 2 REDO
                specialDrop = new MobLoot(mob,ItemBase.getItemBase(252132),true);
                mob.setFirstName("Draco The Blood-Mage");
                break;
            case 15929: //human 3 GOOD
                specialDrop = new MobLoot(mob,ItemBase.getItemBase(252133),true);
                mob.setFirstName("Atlantes The Blood-Mage");
                break;
        }
        if(specialDrop != null) {
            mob.setLevel((short) 65);
            mob.setSpawnTime(10800);
            mob.healthMax = (7500);
            mob.setHealth(7500);
            ChatSystemMsg chatMsg = new ChatSystemMsg(null, mob.getName() + " in " + mob.getParentZone().getName() + " has found the " + specialDrop.getName() + ". Are you tough enough to take it?");
            chatMsg.setMessageType(10);
            chatMsg.setChannel(Enum.ChatChannelType.SYSTEM.getChannelID());
            DispatchMessage.dispatchMsgToAll(chatMsg);
            mob.getCharItemManager().addItemToInventory(specialDrop);
            mob.setResists(new Resists("Dropper"));
        }
    }

    public static MobLoot getGenTableItem(int genTableID, AbstractCharacter mob) {

        if (mob == null || _genTables.containsKey(genTableID) == false)
            return null;

        MobLoot outItem;

        int genRoll = ThreadLocalRandom.current().nextInt(1,94 + 1);

        GenTableEntry selectedRow = GenTableEntry.rollTable(genTableID, genRoll, 1.0f);

        if (selectedRow == null)
            return null;

        int itemTableId = selectedRow.itemTableID;

        if (_itemTables.containsKey(itemTableId) == false)
            return null;

        //gets the 1-320 roll for this mob
        int itemTableRoll = 0;
        int objectType = mob.getObjectType().ordinal();
        if(mob.getObjectType().ordinal() == 52) { //52 = player character
            itemTableRoll = ThreadLocalRandom.current().nextInt(1,320 + 1);
        } else{
            itemTableRoll = TableRoll(mob.level);
        }
        ItemTableEntry tableRow = ItemTableEntry.rollTable(itemTableId, itemTableRoll);
        if (tableRow == null)
            return null;

        int itemUUID = tableRow.cacheID;

        if (itemUUID == 0)
            return null;

        if (ItemBase.getItemBase(itemUUID).getType().ordinal() == Enum.ItemType.RESOURCE.ordinal()) {
            int chance = ThreadLocalRandom.current().nextInt(1,101);
            if(chance > 10)
                return null;
            int amount = ThreadLocalRandom.current().nextInt((int)(tableRow.minSpawn * 0.5f), (int)((tableRow.maxSpawn + 1) * 0.5f));
            return new MobLoot(mob, ItemBase.getItemBase(itemUUID), amount, false);
        }

        outItem = new MobLoot(mob, ItemBase.getItemBase(itemUUID), false);
        Enum.ItemType outType = outItem.getItemBase().getType();


        if(selectedRow.pModTable != 0){
            try {
                outItem = GeneratePrefix(mob, outItem, genTableID, genRoll);
                outItem.setIsID(false);
            } catch (Exception e) {
                Logger.error("Failed to GeneratePrefix for item: " + outItem.getName());
            }
        }
        if(selectedRow.sModTable != 0){
            try {
                outItem = GenerateSuffix(mob, outItem, genTableID, genRoll);
                outItem.setIsID(false);
            } catch (Exception e) {
                Logger.error("Failed to GenerateSuffix for item: " + outItem.getName());
            }
        }
        return outItem;
    }

    private static MobLoot GeneratePrefix(AbstractCharacter mob, MobLoot inItem, int genTableID, int genRoll) {

        GenTableEntry selectedRow = GenTableEntry.rollTable(genTableID, genRoll, 1.0f);

        if (selectedRow == null)
            return inItem;

        int prefixroll = ThreadLocalRandom.current().nextInt(1, 100 + 1);

        ModTypeTableEntry prefixTable = ModTypeTableEntry.rollTable(selectedRow.pModTable, prefixroll);

        if (prefixTable == null)
            return inItem;
        int prefixTableRoll = 0;
        if(mob.getObjectType().ordinal() == 52) {
            prefixTableRoll = ThreadLocalRandom.current().nextInt(1,320 + 1);
        } else{
            prefixTableRoll = TableRoll(mob.level);
        }
        ModTableEntry prefixMod = ModTableEntry.rollTable(prefixTable.modTableID, prefixTableRoll);

        if (prefixMod == null)
            return inItem;

        if (prefixMod.action.length() > 0) {
            inItem.setPrefix(prefixMod.action);
            inItem.addPermanentEnchantment(prefixMod.action, 0, prefixMod.level, true);
        }

        return inItem;
    }

    private static MobLoot GenerateSuffix(AbstractCharacter mob, MobLoot inItem, int genTableID, int genRoll) {

        GenTableEntry selectedRow = GenTableEntry.rollTable(genTableID, genRoll, 1.0f);

        if (selectedRow == null)
            return inItem;

        int suffixRoll = ThreadLocalRandom.current().nextInt(1, 100 + 1);

        ModTypeTableEntry suffixTable = ModTypeTableEntry.rollTable(selectedRow.sModTable, suffixRoll);

        if (suffixTable == null)
            return inItem;
        int suffixTableRoll = 0;
        if(mob.getObjectType().ordinal() == 52) {
            suffixTableRoll = ThreadLocalRandom.current().nextInt(1,320 + 1);
        } else{
            suffixTableRoll = TableRoll(mob.level);
        }
        ModTableEntry suffixMod = ModTableEntry.rollTable(suffixTable.modTableID, suffixTableRoll);

        if (suffixMod == null)
            return inItem;

        if (suffixMod.action.length() > 0) {
            inItem.setSuffix(suffixMod.action);
            inItem.addPermanentEnchantment(suffixMod.action, 0, suffixMod.level, false);
        }

        return inItem;
    }

    public static int TableRoll(int mobLevel) {

        if (mobLevel > 65)
            mobLevel = 65;

        int max = (int) (4.882 * mobLevel + 127.0);

        if (max > 319)
            max = 319;

        int min = (int) (4.469 * mobLevel - 3.469);

        if (min < 70)
            min = 70;

        int roll = ThreadLocalRandom.current().nextInt(min, max + 1);

        return roll;
    }

    public static void GenerateGoldDrop(Mob mob, BootySetEntry bse) {

        int chanceRoll = ThreadLocalRandom.current().nextInt(1, 100 + 1);

        //early exit, failed to hit minimum chance roll

        if (chanceRoll > bse.dropChance)
            return;

        //determine and add gold to mob inventory

        int high = (int)(bse.highGold * NORMAL_GOLD_RATE);
        int low = (int)(bse.lowGold * NORMAL_GOLD_RATE);
        int gold = ThreadLocalRandom.current().nextInt(low, high);

        if (gold > 0) {
            MobLoot goldAmount = new MobLoot(mob, gold);
            mob.getCharItemManager().addItemToInventory(goldAmount);
        }

    }

    public static void GenerateLootDrop(Mob mob, int tableID) {

        try {
            if(mob.parentZone.getSafeZone() == 1) {
                return;
            }
            MobLoot toAdd = getGenTableItem(tableID, mob);
            if(toAdd.getItemBase().getType().equals(Enum.ItemType.CONTRACT) || toAdd.getItemBase().getType().equals(Enum.ItemType.RUNE))
                return;//block all contracts and runes that drop outside the confines of the new system

            if (toAdd != null) {
                toAdd.setIsID(true);
                mob.getCharItemManager().addItemToInventory(toAdd);
            }
        } catch (Exception e) {
            //TODO chase down loot generation error, affects roughly 2% of drops
        }
    }

    public static void GenerateEquipmentDrop(Mob mob) {

        if(mob.parentZone.getSafeZone() == 1) {
            return;
        }

        //do equipment here
        int dropCount = 0;
        if (mob.getEquip() != null)
            for (MobEquipment me : mob.getEquip().values()) {

                if (me.getDropChance() == 0)
                    continue;

                float equipmentRoll = ThreadLocalRandom.current().nextInt(1, 100 + 1);
                float dropChance = me.getDropChance() * 100;
                if (equipmentRoll > dropChance)
                    continue;
                ItemBase genericIB = me.getItemBase();
                if(genericIB.isVorg()){
                    if(genericIB.isClothArmor()){
                        //get random cloth piece
                        genericIB = getRandomVorgCloth();//ItemBase.getItemBase(vorg_cloth_uuids.get(ThreadLocalRandom.current().nextInt(0,vorg_cloth_uuids.size() - 1)));
                    } else if(genericIB.isHeavyArmor()){
                        //get random heavy armor piece
                        genericIB = getRandomVorgHA();//ItemBase.getItemBase(vorg_ha_uuids.get(ThreadLocalRandom.current().nextInt(0,vorg_ha_uuids.size() - 1)));
                    } else if(genericIB.isMediumArmor()){
                        //get random medium armor piece
                        genericIB = getRandomVorgMA();//ItemBase.getItemBase(vorg_ma_uuids.get(ThreadLocalRandom.current().nextInt(0,vorg_ma_uuids.size() - 1)));
                    } else if(genericIB.isLightArmor()){
                        //get random light armor piece
                        genericIB = getRandomVorgLA();//ItemBase.getItemBase(vorg_la_uuids.get(ThreadLocalRandom.current().nextInt(0,vorg_la_uuids.size() - 1)));
                    }
                    mob.spawnTime = ThreadLocalRandom.current().nextInt(300,2700);
                }
                    MobLoot ml = new MobLoot(mob, genericIB, false);

                if (ml != null && dropCount < 1 && genericIB.isVorg() == false) {
                    ml.setIsID(true);
                    ml.setDurabilityCurrent((short) (ml.getDurabilityCurrent() - ThreadLocalRandom.current().nextInt(5) + 1));
                    mob.getCharItemManager().addItemToInventory(ml);
                    dropCount = 1;
                    //break; // Exit on first successful roll.
                }
                if(ml != null && genericIB.isVorg() && mob.getMobBaseID() != 14062){
                    ml.setIsID(true);
                    ml.setDurabilityCurrent(ml.getDurabilityMax());
                    mob.getCharItemManager().addItemToInventory(ml);
                }
            }
    }

    public static void GenerateInventoryDrop(Mob mob, BootySetEntry bse) {

        if(ItemBase.getItemBase(bse.itemBase).isDiscRune()) {
            if(!Mob.disciplineDroppers.contains(mob))
                Mob.disciplineDroppers.add(mob);

            mob.setResists(new Resists("Dropper"));
            ChatSystemMsg chatMsg = new ChatSystemMsg(null, mob.getName() + " in " + mob.getParentZone().getName() + " has found the " + ItemBase.getItemBase(bse.itemBase).getName() + ". Are you tough enough to take it?");
            chatMsg.setMessageType(10);
            chatMsg.setChannel(Enum.ChatChannelType.SYSTEM.getChannelID());
            DispatchMessage.dispatchMsgToAll(chatMsg);
        }

        //if((bse.itemBase == 3040 || bse.itemBase == 3021) && mob.level < 80){
        //    chance = 100;
        //}

        if(mob.parentZone.getSafeZone() == 1) {
            return;
        }

        int chanceRoll = ThreadLocalRandom.current().nextInt(1, 99);

        //early exit, failed to hit minimum chance roll

        if (chanceRoll > bse.dropChance)
            return;

        MobLoot lootItem = new MobLoot(mob, ItemBase.getItemBase(bse.itemBase), true);

        if (lootItem != null)
            mob.getCharItemManager().addItemToInventory(lootItem);
    }

    public static void peddleFate(PlayerCharacter playerCharacter, Item gift) {

        //get table ID for the itembase ID

        int tableID = 0;

        if (_bootySetMap.get(gift.getItemBaseID()) != null)
            tableID = _bootySetMap.get(gift.getItemBaseID()).get(ThreadLocalRandom.current().nextInt(_bootySetMap.get(gift.getItemBaseID()).size())).genTable;

        if (tableID == 0)
            return;

        //get the character item manager

        CharacterItemManager itemMan = playerCharacter.getCharItemManager();

        if (itemMan == null)
            return;

        //check if player owns the gift he is trying to open

        if (itemMan.doesCharOwnThisItem(gift.getObjectUUID()) == false)
            return;

        //roll 1-100 for the gen table selection

        int genRoll = ThreadLocalRandom.current().nextInt(1, 100 + 1);
        GenTableEntry selectedRow = GenTableEntry.rollTable(tableID, genRoll, LootManager.NORMAL_DROP_RATE);

        if(selectedRow == null)
            return;

        //roll 220-320 for the item table selection

        int itemRoll = ThreadLocalRandom.current().nextInt(220, 320 + 1);
        ItemTableEntry selectedItem = ItemTableEntry.rollTable(selectedRow.itemTableID, itemRoll);

        if (selectedItem == null)
            return;

        //create the item from the table, quantity is always 1

        MobLoot winnings = new MobLoot(playerCharacter, ItemBase.getItemBase(selectedItem.cacheID), 1, false);

        if (winnings == null)
            return;

        //early exit if the inventory of the player will not old the item

        if (itemMan.hasRoomInventory(winnings.getItemBase().getWeight()) == false) {
            ErrorPopupMsg.sendErrorPopup(playerCharacter, 21);
            return;
        }

        //determine if the winning item needs a prefix

        if(selectedRow.pModTable != 0){
            int prefixRoll = ThreadLocalRandom.current().nextInt(220,320 + 1);
            ModTableEntry prefix = ModTableEntry.rollTable(selectedRow.pModTable, prefixRoll);
            if(prefix != null)
                winnings.addPermanentEnchantment(prefix.action, 0, prefix.level, true);
        }

        //determine if the winning item needs a suffix

        if(selectedRow.sModTable != 0){
            int suffixRoll = ThreadLocalRandom.current().nextInt(220,320 + 1);
            ModTableEntry suffix = ModTableEntry.rollTable(selectedRow.sModTable, suffixRoll);
            if (suffix != null)
                winnings.addPermanentEnchantment(suffix.action, 0, suffix.level, true);
        }
        winnings.setIsID(true);

        //remove gift from inventory

        itemMan.consume(gift);

        //add winnings to player inventory

        Item playerWinnings = winnings.promoteToItem(playerCharacter);
        itemMan.addItemToInventory(playerWinnings);
        itemMan.updateInventory();
    }

    public static MobLoot rollForContract(int table, Mob mob){

        int roll = 99;
        if (table == 1900 || table == 1500)
            roll = 73;
        GenTableEntry selectedRow = GenTableEntry.rollTable(table, roll, 1.0f);
        if (selectedRow == null)
            return null;

        int itemTableId = selectedRow.itemTableID;

        if (_itemTables.containsKey(itemTableId) == false)
            return null;
        ItemTableEntry tableRow = ItemTableEntry.rollTable(itemTableId, ThreadLocalRandom.current().nextInt(75,321));
        if (tableRow == null)
            return null;

        int itemUUID = tableRow.cacheID;

        if (itemUUID == 0)
            return null;

        MobLoot outItem = new MobLoot(mob, ItemBase.getItemBase(itemUUID), false);
        if(outItem != null) {
            mob.contractCounter = ThreadLocalRandom.current().nextInt(200);
            ChatSystemMsg chatMsg = new ChatSystemMsg(null, mob.getName() + " looks like he found something special");
            chatMsg.setMessageType(10);
            chatMsg.setChannel(Enum.ChatChannelType.SYSTEM.getChannelID());
            DispatchMessage.dispatchMsgToInterestArea(mob,chatMsg, Enum.DispatchChannel.PRIMARY, MBServerStatics.CHARACTER_LOAD_RANGE,false,false);
            return outItem;
        }
        return null;
    }
    public static MobLoot rollForRune(int table, Mob mob){
        int roll = 97;
        if(table == 1900 || table == 1500){
            roll = 77;
        }
        GenTableEntry selectedRow = GenTableEntry.rollTable(table, roll, 1.0f);
        if (selectedRow == null)
            return null;

        int itemTableId = selectedRow.itemTableID;

        if (_itemTables.containsKey(itemTableId) == false)
            return null;
        ItemTableEntry tableRow = ItemTableEntry.rollTable(itemTableId, ThreadLocalRandom.current().nextInt(75,321));
        if (tableRow == null)
            return null;

        int itemUUID = tableRow.cacheID;

        if (itemUUID == 0)
            return null;

        MobLoot outItem = new MobLoot(mob, ItemBase.getItemBase(itemUUID), false);
        if(outItem != null) {
            mob.runeCounter = ThreadLocalRandom.current().nextInt(200);
            ChatSystemMsg chatMsg = new ChatSystemMsg(null, mob.getName() + " looks like he found something special");
            chatMsg.setMessageType(10);
            chatMsg.setChannel(Enum.ChatChannelType.SYSTEM.getChannelID());
            DispatchMessage.dispatchMsgToInterestArea(mob,chatMsg, Enum.DispatchChannel.PRIMARY, MBServerStatics.CHARACTER_LOAD_RANGE,false,false);
            return outItem;
        }
        return null;
    }
    public static MobLoot rollForGlass( Mob mob){
        ItemTableEntry tableRow = ItemTableEntry.rollTable(126, ThreadLocalRandom.current().nextInt(220,321));
        if (tableRow == null)
            return null;

        int itemUUID = tableRow.cacheID;

        if (itemUUID == 0)
            return null;

        MobLoot outItem = new MobLoot(mob, ItemBase.getItemBase(itemUUID), false);
        if(outItem != null)
            return outItem;
        return null;
    }

    public static ItemBase getRandomVorgCloth(){
        int random = ThreadLocalRandom.current().nextInt(100);
        if(random < 20)
            return ItemBase.getItemBase(27600);
        if(random > 20 && random < 40)
            return ItemBase.getItemBase(188700);
        if(random > 40 && random < 60)
            return ItemBase.getItemBase(188720);
        if(random > 60 && random < 80)
            return ItemBase.getItemBase(189550);
        if(random > 80)
            return ItemBase.getItemBase(189560);
        return null;
    }

    public static ItemBase getRandomVorgLA(){
        int random = ThreadLocalRandom.current().nextInt(160);
        if(random < 20)
            return ItemBase.getItemBase(27550);
        if(random > 20 && random < 40)
            return ItemBase.getItemBase(27560);
        if(random > 40 && random < 60)
            return ItemBase.getItemBase(189100);
        if(random > 60 && random < 80)
            return ItemBase.getItemBase(189110);
        if(random > 80 && random < 100)
            return ItemBase.getItemBase(189120);
        if(random > 100 && random < 120)
            return ItemBase.getItemBase(189130);
        if(random > 120 && random < 140)
            return ItemBase.getItemBase(189140);
        if(random > 140)
            return ItemBase.getItemBase(189150);
        return null;
    }
    public static ItemBase getRandomVorgMA(){
        int random = ThreadLocalRandom.current().nextInt(160);
        if(random < 20)
            return ItemBase.getItemBase(27570);
        if(random > 20 && random < 40)
            return ItemBase.getItemBase(188900);
        if(random > 40 && random < 60)
            return ItemBase.getItemBase(188910);
        if(random > 60 && random < 80)
            return ItemBase.getItemBase(188920);
        if(random > 80 && random < 100)
            return ItemBase.getItemBase(188930);
        if(random > 100 && random < 120)
            return ItemBase.getItemBase(188940);
        if(random > 120 && random < 140)
            return ItemBase.getItemBase(188950);
        if(random > 140)
            return ItemBase.getItemBase(189500);
        return null;
    }
    public static ItemBase getRandomVorgHA(){
        int random = ThreadLocalRandom.current().nextInt(180);
        if(random < 20)
            return ItemBase.getItemBase(27580);
        if(random > 20 && random < 40)
            return ItemBase.getItemBase(27590);
        if(random > 40 && random < 60)
            return ItemBase.getItemBase(188500);
        if(random > 60 && random < 80)
            return ItemBase.getItemBase(188510);
        if(random > 80 && random < 100)
            return ItemBase.getItemBase(188520);
        if(random > 100 && random < 120)
            return ItemBase.getItemBase(188530);
        if(random > 120 && random < 140)
            return ItemBase.getItemBase(188540);
        if(random > 140 && random < 160)
            return ItemBase.getItemBase(188550);
        if(random > 160)
            return ItemBase.getItemBase(189510);
        return null;
    }
}
