// • ▌ ▄ ·.  ▄▄▄·  ▄▄ • ▪   ▄▄· ▄▄▄▄·  ▄▄▄·  ▐▄▄▄  ▄▄▄ .
// ·██ ▐███▪▐█ ▀█ ▐█ ▀ ▪██ ▐█ ▌▪▐█ ▀█▪▐█ ▀█ •█▌ ▐█▐▌·
// ▐█ ▌▐▌▐█·▄█▀▀█ ▄█ ▀█▄▐█·██ ▄▄▐█▀▀█▄▄█▀▀█ ▐█▐ ▐▌▐▀▀▀
// ██ ██▌▐█▌▐█ ▪▐▌▐█▄▪▐█▐█▌▐███▌██▄▪▐█▐█ ▪▐▌██▐ █▌▐█▄▄▌
// ▀▀  █▪▀▀▀ ▀  ▀ ·▀▀▀▀ ▀▀▀·▀▀▀ ·▀▀▀▀  ▀  ▀ ▀▀  █▪ ▀▀▀
//      Magicbane Emulator Project © 2013 - 2022
//                www.magicbane.com
package engine.gameManager;
// Defines static methods which comprise the magicbane
// building maintenance system.
import engine.Enum;
import engine.objects.*;
import org.pmw.tinylog.Logger;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.concurrent.ConcurrentHashMap;
public enum MaintenanceManager {
    MAINTENANCEMANAGER;
    public static void setMaintDateTime(Building building, LocalDateTime maintDate) {
        building.maintDateTime = maintDate;
        DbManager.BuildingQueries.updateMaintDate(building);
    }
    public static void dailyMaintenance() {
        Logger.info("Maintenance has started");
        // Run maintenance on player buildings
        if (ConfigManager.MB_WORLD_MAINTENANCE.getValue().equalsIgnoreCase("true"))
            processMaintenance();
        else
            Logger.info("Maintenance Costings: DISABLED");
        Logger.info("Maintenance has completed!");
    }
    public static void processMaintenance() {
        //create list of all cities
        ConcurrentHashMap<Integer, AbstractGameObject> worldCities = DbManager.getMap(Enum.GameObjectType.City);
        //loop all cities
        for (AbstractGameObject ago : worldCities.values()) {
            if (ago.getObjectType().equals(Enum.GameObjectType.City)) {
                City city = (City) ago;
                if(city == null || !city.getParent().isPlayerCity())
                    continue;
                Building tol = city.getTOL();
                if(tol == null)
                    continue;
                LocalDateTime maintenanceDueDate = tol.maintDateTime.withHour(1).withMinute(0).withSecond(0);
                LocalDateTime now = LocalDateTime.now();
                if(now.isAfter(maintenanceDueDate))
                    processTolMaintenance(tol, city.getWarehouse());
            }
        }
    }
    public static void processTolMaintenance(Building tol, Warehouse warehouse){
        if(tol == null)
            return;
        if(tol.getRank() == 8)
            handleR8(tol,warehouse);
        else
            handleNormal(tol,warehouse);
    }
    public static void handleNormal(Building tol, Warehouse warehouse){
        //handle r7 and lower ToL maintenance
        int goldDue = 3000000;
        //enough on strongbox alone to pay
        if (tol.getStrongboxValue() >= goldDue) {
            tol.maintDateTime = LocalDateTime.now().plusDays(7);
            if(DbManager.BuildingQueries.updateMaintDate(tol)) {
                tol.setStrongboxValue(tol.getStrongboxValue() - goldDue);
            }
            return;
        }
        int newStrongboxValue = tol.getStrongboxValue();
        int newWarehouseGold = 0;
        if(warehouse != null && warehouse.getResources().get(ItemBase.getItemBase(7)) != null) {
            newWarehouseGold = warehouse.getResources().get(ItemBase.getItemBase(7));
        } else{
            //wasnt enough on strongbox and gold in warehouse is empty
            tol.maintDateTime = LocalDateTime.now().plusDays(1);
            if(DbManager.BuildingQueries.updateMaintDate(tol)) {
                tol.destroyOrDerank(null);
            }
            return;
        }
        //some on strongbox to pay
        if (tol.getStrongboxValue() > 0) {
            newStrongboxValue = 0;
            goldDue -= tol.getStrongboxValue();
        }
        if(newWarehouseGold < goldDue){
            //not enough gold to pay, you miss maintenance
            tol.maintDateTime = LocalDateTime.now().plusDays(1);
            if(DbManager.BuildingQueries.updateMaintDate(tol)) {
                tol.destroyOrDerank(null);
            }
            return;
        }
        newWarehouseGold -= goldDue;
        tol.maintDateTime = LocalDateTime.now().plusDays(7);
        if(DbManager.BuildingQueries.updateMaintDate(tol) && DbManager.WarehouseQueries.updateGold(warehouse,newWarehouseGold)) {
            warehouse.getResources().put(ItemBase.getItemBase(7), newWarehouseGold);
            tol.setStrongboxValue(newStrongboxValue);
        }
    }
    public static void handleR8(Building tol, Warehouse warehouse){
        //handle r8 ToL maintenance
        //cannot pay r8 maintenance without a warehouse
        if(warehouse == null && DbManager.BuildingQueries.updateMaintDate(tol)) {
            tol.destroyOrDerank(null);
            tol.maintDateTime = LocalDateTime.now().plusDays(1);
            return;
        }
        //handle resource processing
        int goldDue = 3000000;
        int galvorDue = 5;
        int wormwoodDue = 5;
        int stoneDue = 5000;
        int lumberDue = 5000;
        int goldStrongBox = tol.getStrongboxValue();
        int goldWarehouse = 0;
        int galvorWarehouse;
        int wormwoodWarehouse;
        int stoneWarehouse;
        int lumberWarehouse;
        if(warehouse.getResources().get(Warehouse.galvorIB) != null) {
            galvorWarehouse = warehouse.getResources().get(Warehouse.galvorIB);
        }else {
            tol.maintDateTime = LocalDateTime.now().plusDays(1);
            if(DbManager.BuildingQueries.updateMaintDate(tol)) {
                tol.destroyOrDerank(null);
            }
            return;
        }
        if(warehouse.getResources().get(Warehouse.stoneIB) != null) {
            stoneWarehouse = warehouse.getResources().get(Warehouse.stoneIB);
        }else {
            tol.maintDateTime = LocalDateTime.now().plusDays(1);
            if(DbManager.BuildingQueries.updateMaintDate(tol)) {
                tol.destroyOrDerank(null);
            }
            return;
        }
        if(warehouse.getResources().get(Warehouse.wormwoodIB) != null) {
            wormwoodWarehouse = warehouse.getResources().get(Warehouse.wormwoodIB);
        }else {
            tol.maintDateTime = LocalDateTime.now().plusDays(1);
            if(DbManager.BuildingQueries.updateMaintDate(tol)) {
                tol.destroyOrDerank(null);
            }
            return;
        }
        if(warehouse.getResources().get(Warehouse.lumberIB) != null) {
            lumberWarehouse = warehouse.getResources().get(Warehouse.lumberIB);
        }else {
            tol.maintDateTime = LocalDateTime.now().plusDays(1);
            if(DbManager.BuildingQueries.updateMaintDate(tol)) {
                tol.destroyOrDerank(null);
            }
            return;
        }
        boolean canPay = true;
        if(goldStrongBox >= goldDue){
            goldStrongBox -= goldDue;
            goldDue = 0;
        }
        if (tol.getStrongboxValue() > 0) {
            goldStrongBox = 0;
            goldDue -= tol.getStrongboxValue();
        }
        if(warehouse.getResources().get(Warehouse.goldIB) != null) {
            goldWarehouse = warehouse.getResources().get(Warehouse.goldIB);
        }else if(goldDue > 0){
            tol.maintDateTime = LocalDateTime.now().plusDays(1);
            if(DbManager.BuildingQueries.updateMaintDate(tol)) {
                tol.destroyOrDerank(null);
            }
            return;
        }
        if(wormwoodDue > wormwoodWarehouse)
            canPay = false;
        if(galvorDue > galvorWarehouse)
            canPay = false;
        if(lumberDue > lumberWarehouse)
            canPay = false;
        if(stoneDue > stoneWarehouse)
            canPay = false;
        if(goldDue > goldWarehouse)
            canPay = false;
        if(!canPay){
            tol.maintDateTime = LocalDateTime.now().plusDays(1);
            if(DbManager.BuildingQueries.updateMaintDate(tol)) {
                tol.destroyOrDerank(null);
            }
            return;
        }
        tol.setStrongboxValue(goldStrongBox);
        if(DbManager.WarehouseQueries.updateGold(warehouse,goldWarehouse - goldDue)){
            if(DbManager.WarehouseQueries.updateStone(warehouse,stoneWarehouse - stoneDue)){
                if(DbManager.WarehouseQueries.updateLumber(warehouse,lumberWarehouse - lumberDue)){
                    if(DbManager.WarehouseQueries.updateGalvor(warehouse,galvorWarehouse - galvorDue)){
                        if(DbManager.WarehouseQueries.updateWormwood(warehouse,wormwoodWarehouse - wormwoodDue)){
                            tol.maintDateTime = LocalDateTime.now().plusDays(1);
                            if(DbManager.BuildingQueries.updateMaintDate(tol)) {
                                return;
                            }
                        }
                    }
                }
            }
        }
    }
}